pro ssw_service, query, qfile=qfile, $
   foreground=foreground , background=background, debug=debug

;+
;   Name: ssw_service
;
;   Purpose: manage SSW/WWW services
;
;   Input Paramters:
;      query - POST or GET query stream
;
;   Keyword Parameters:
;      qfile - POST or GET query file (alternative to query input)
;      foreground - if set, run in foreground (override QUERY setting)
;      background - if set, background (add to query pending 
;
;   History:
;      25-October-2001 - S.L.Freeland
;       7-November-2001 - S.L.Freeland - enable plot_map/plot_goes
;                         optional client display
;-


debug=keyword_set(debug)
set_logenv,'path_http','/net/diapason/www1/htdocs/solarsoft/ssw_client'
set_logenv,'top_http','http://www.lmsal.com/solarsoft/ssw_client'

case 1 of 
   data_chk(qfile,/string): begin
      box_message,['QFILE: '+qfile,rd_tfile(qfile)] 
      info=url_decode(qfile=qfile)
      break_file,qfile,ll,pp,ff,ee,vv
      
      outdir=concat_dir('$path_http','data')
      help,outdir,http_names(outdir)
   endcase
   data_chk(query,/string): info=query
   else: begin 
      box_message,'Require QUERY string or QUERY file
      return
   endcase
endcase

background=gt_tagval(info,/ssw_service_execution,missing='') eq 'background' $
   or keyword_set(background)
foreground=keyword_set(foreground) or (1-background)

help,info,/str
service=gt_tagval(info,/SSW_SERVICE,missing='')
if total(strpos(service,' ')) ge 0 then $
   service=strlowcase(ssw_strsplit(service,' ',/head))
netdisp=gt_tagval(info,/NETSCAPE_DISPLAY, missing='')

s1=gt_tagval(info,/param1_sample,missing='')
s2=gt_tagval(info,/param2_sample,missing='')

p1=([gt_tagval(info,/param1,missing=''),s1])(s1 ne '')

p2=([gt_tagval(info,/param2,missing=''),s2])(s2 ne '')
 
extra=strcompress(strlowcase(gt_tagval(info,/EXTRA,missing='')),/remove)

textra=extra+','

if strpos(textra,'xsize=') ne -1 then $
   xsize=strextract(textra,'xsize=',',') else xsize='600'

if strpos(textra,'ysize=') ne -1 then $
   ysize=strextract(textra,'ysize=',',') else ysize=xsize

break_url,p1,server1,path1,files1,ftp=ftp,http=http
break_url,p2,server2,path2,files2,ftp=ftp,http=http

post2=gt_tagval(info,/URL_DESTINATION,missing='')

help,service(0),background,p1,p2, xsize, ysize

ps1='param1='
ps2='param2='
ext='extra='

if debug then stop,'before background/forground case'

if background then begin
   box_message,'background task'
   outfile=concat_dir('$path_http','www_pending')
   file_append,outfile,rd_tfile(qfile),/new
endif else begin 

   box_message,'foreground'
   case service(0) of 
      'goes_event': begin
         found=0
         estring='ssws_gevevent,p1, p2, ot1, ot2, found=found, /biggest' $
              + (["",',' + extra])(extra ne '')
         estat=execute(estring)
         if found then begin 
            qfile=ps1+url_encode(ot1(0))+'&'+ps2+url_encode(ot2(0))
         endif
      endcase
      'plot_map': begin
          ssw_url2data,p1,index,data
          if data_chk(index,/struct) then begin 
             set_plot,'z'
             wdef,xx,str2number(xsize)>256,str2number(ysize)>256,/zbuffer
             loadct,0
             index2map,index,data,map
             estring='plot_map,map' + (["",',' + extra])(extra ne '') 
             estat=execute(estring)
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=http_names(ofile)
          endif
      endcase

      'coalign': begin
      endcase

      'goes_xrays': begin
          set_plot,'z'
          estring="plot_goes,'"+ p1+"','" +p2+"'"
          estring=estring+ (["",',' + extra])(extra ne '')
             estat=execute(estring)
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=http_names(ofile)

      endcase

      'goes_particles': begin 
          set_plot,'z'
          estring="plot_goesp,'"+ p1+"','" +p2+"'"
          estring=estring+ (["",',' + extra])(extra ne '')
             estat=execute(estring) 
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=http_names(ofile)
      endcase

      'plot_ace': begin 
          set_plot,'z'
          estring="plot_ace,'"+ p1+"','" +p2+"'"
          estring=estring+ (["",',' + extra])(extra ne '')
            estat=execute(estring) 
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=http_names(ofile)
      endcase

      else: begin
      endcase
   endcase
   if post2 ne '' then begin
      box_message,'Posting result...'
      if debug then stop,'post'
      box_message,['POST2> ' + post2,'QUERY> ' + qfile]
      ssw_post_query,post2,qfile, status=status      
      if netdisp ne '' then begin 
         estring='netscape_control,"'+qfile+'",/openurl,display="'+netdisp+'"'
         estat=execute(estring)
      endif
   endif
endelse

return
end
